%{
#include <stdio.h>
#include <string.h>
#include "opcode.h"

FILE *yyin;

void yyerror(const char *str)
{
	fprintf(stderr ,"error: %s\n", str);
}

int yywrap()
{
	return 1;
}

FILE *fin, *fout;

int main(int argc, char *argv[])
{	
	if(argc == 2)
	{
		fin = fopen(argv[1], "r");
		fout = fopen("code.out", "w");
	}
	else
		fin = NULL;
	
	if(!fin||!fout)
	{
		printf("usage: %s input_file\n", argv[0]);
		return 1;
	}
	
	
	
	if(fin) yyin = fin;
	yyparse();
	
	fclose(fin);
	fclose(fout);
	
	return 0;
}

%}

%token ADD
%token SUB
%token MUL
%token DIV
%token SLL
%token SRL
%token SRR
%token AND
%token OR
%token XOR
%token NOT

%token ADDI
%token SUBI
%token MULI
%token DIVI
%token ANDI
%token ORI
%token XORI

%token FADD
%token FSUB
%token FMUL
%token FDIV
%token ITOF
%token FTOI
%token FABS
%token FNEG
%token FLOOR
%token FSQRT

%token NOP
%token HLT
%token RD
%token WR

%token LOAD
%token STORE

%token JUMP
%token JAL
%token JR

%token BL
%token BE
%token BLE
%token BGE
%token BNE
%token BG
%token BLZ
%token BEZ
%token BLEZ
%token BGEZ
%token BNEZ
%token BGZ

%token FBL
%token FBE
%token FBLE
%token FBGE
%token FBNE
%token FBG
%token FBLZ
%token FBEZ
%token FBLEZ
%token FBGEZ
%token FBNEZ
%token FBGZ

%token REG
%token NUM
%token NL

%start operations

%%

operations
	:
	| operations operation NL { fprintf(fout, "0x%08X\n", $2); }
	;

operation
	: ADD REG REG REG { $$ = OP_ALU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_ADD; }
	| SUB REG REG REG { $$ = OP_ALU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_SUB; }
	| MUL REG REG REG { $$ = OP_ALU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_MUL; }
	| DIV REG REG REG { $$ = OP_ALU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_DIV; }
	| SLL REG REG NUM { $$ = OP_ALU << 26 | $3 << 21 | $4 << 11 | (0x1F & $2) << 6 | FNCT_SLL; }
	| SRL REG REG NUM { $$ = OP_ALU << 26 | $3 << 21 | $4 << 11 | (0x1F & $2) << 6 | FNCT_SRL; }
	| SRR REG REG NUM { $$ = OP_ALU << 26 | $3 << 21 | $4 << 11 | (0x1F & $2) << 6 | FNCT_SRR; }
	| AND REG REG REG { $$ = OP_ALU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_AND; }
	| OR  REG REG REG { $$ = OP_ALU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_OR ; }
	| XOR REG REG REG { $$ = OP_ALU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_XOR; }
	| NOT REG REG REG { $$ = OP_ALU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_NOT; }

	| ADDI REG REG NUM { $$ = OP_ADDI << 26 | $3 << 21 | $2 << 16 | 0xFFFF & $4;}
	| SUBI REG REG NUM { $$ = OP_SUBI << 26 | $3 << 21 | $2 << 16 | 0xFFFF & $4;}
	| MULI REG REG NUM { $$ = OP_MULI << 26 | $3 << 21 | $2 << 16 | 0xFFFF & $4;}
	| DIVI REG REG NUM { $$ = OP_DIVI << 26 | $3 << 21 | $2 << 16 | 0xFFFF & $4;}
	| ANDI REG REG NUM { $$ = OP_ANDI << 26 | $3 << 21 | $2 << 16 | 0xFFFF & $4;}
	| ORI  REG REG NUM { $$ = OP_ORI  << 26 | $3 << 21 | $2 << 16 | 0xFFFF & $4;}
	| XORI REG REG NUM { $$ = OP_XORI << 26 | $3 << 21 | $2 << 16 | 0xFFFF & $4;}

	| FADD REG REG REG { $$ = OP_FPU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_FADD; }
	| FSUB REG REG REG { $$ = OP_FPU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_FSUB; }
	| FMUL REG REG REG { $$ = OP_FPU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_FMUL; }
	| FDIV REG REG REG { $$ = OP_FPU << 26 | $3 << 21 | $4 << 16 | $2 << 11 | FNCT_FDIV; }
	| ITOF  REG REG    { $$ = OP_FPU << 26 | $3 << 21 | $2 << 16 | FNCT_ITOF ; }
	| FTOI  REG REG    { $$ = OP_FPU << 26 | $3 << 21 | $2 << 16 | FNCT_FTOI ; }
	| FABS  REG REG    { $$ = OP_FPU << 26 | $3 << 21 | $2 << 16 | FNCT_FABS ; }
	| FNEG  REG REG    { $$ = OP_FPU << 26 | $3 << 21 | $2 << 16 | FNCT_FNEG ; }
	| FLOOR REG REG    { $$ = OP_FPU << 26 | $3 << 21 | $2 << 16 | FNCT_FLOOR; }
	| FSQRT REG REG    { $$ = OP_FPU << 26 | $3 << 21 | $2 << 16 | FNCT_FSQRT; }

	| NOP    { $$ = OP_SYS << 26 | FNCT_NOP; }
	| HLT    { $$ = OP_SYS << 26 | FNCT_HLT; }
	| RD REG { $$ = OP_SYS << 26 | $2 << 11 | FNCT_RD; }
	| WR REG { $$ = OP_SYS << 26 | $2 << 21 | FNCT_WR; }
	
	| LOAD  REG REG NUM { $$ = OP_LOAD  << 26 | $3 << 21 | $2 << 16 | 0xFFFF & $4; }
	| STORE REG REG NUM { $$ = OP_STORE << 26 | $3 << 21 | $2 << 16 | 0xFFFF & $4; }
	
	| JUMP NUM { $$ = OP_JUMP << 26 | 0x3FFFFFF & $2; }
	| JAL  NUM { $$ = OP_JAL  << 26 | 0x3FFFFFF & $2; }
	| JR REG   { $$ = OP_JR   << 26 | $2 <<21; }
	
	| BL   REG REG NUM { $$ = OP_BL   << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| BE   REG REG NUM { $$ = OP_BE   << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| BLE  REG REG NUM { $$ = OP_BLE  << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| BGE  REG REG NUM { $$ = OP_BGE  << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| BNE  REG REG NUM { $$ = OP_BNE  << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| BG   REG REG NUM { $$ = OP_BG   << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| BLZ  REG NUM     { $$ = OP_BLZ  << 26 | $2 << 21 | 0xFFFF & $3; }
	| BEZ  REG NUM     { $$ = OP_BEZ  << 26 | $2 << 21 | 0xFFFF & $3; }
	| BLEZ REG NUM     { $$ = OP_BLEZ << 26 | $2 << 21 | 0xFFFF & $3; }
	| BGEZ REG NUM     { $$ = OP_BGEZ << 26 | $2 << 21 | 0xFFFF & $3; }
	| BNEZ REG NUM     { $$ = OP_BNEZ << 26 | $2 << 21 | 0xFFFF & $3; }
	| BGZ  REG NUM     { $$ = OP_BGZ  << 26 | $2 << 21 | 0xFFFF & $3; }
	
	| FBL   REG REG NUM { $$ = OP_FBL   << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| FBE   REG REG NUM { $$ = OP_FBE   << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| FBLE  REG REG NUM { $$ = OP_FBLE  << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| FBGE  REG REG NUM { $$ = OP_FBGE  << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| FBNE  REG REG NUM { $$ = OP_FBNE  << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| FBG   REG REG NUM { $$ = OP_FBG   << 26 | $2 << 21 | $3 << 16 | 0xFFFF & $4; }
	| FBLZ  REG NUM     { $$ = OP_FBLZ  << 26 | $2 << 21 | 0xFFFF & $3; }
	| FBEZ  REG NUM     { $$ = OP_FBEZ  << 26 | $2 << 21 | 0xFFFF & $3; }
	| FBLEZ REG NUM     { $$ = OP_FBLEZ << 26 | $2 << 21 | 0xFFFF & $3; }
	| FBGEZ REG NUM     { $$ = OP_FBGEZ << 26 | $2 << 21 | 0xFFFF & $3; }
	| FBNEZ REG NUM     { $$ = OP_FBNEZ << 26 | $2 << 21 | 0xFFFF & $3; }
	| FBGZ  REG NUM     { $$ = OP_FBGZ  << 26 | $2 << 21 | 0xFFFF & $3; }
	
	;

%%
